﻿using ApiLog.Act.DAL.Mongo;
using ApiLog.Act.Models.Log;
using ApiLog.Act.Tools;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Web.Http.Controllers;
using System.Web.Http.Filters;

namespace ApiLog.Act.Filters
{
    /// <summary>
    /// 进入数据日志处理-过滤器
    /// </summary>

    public class RequestLogAttribute : System.Web.Http.Filters.ActionFilterAttribute
    {
        #region 初始化

        /// <summary>
        /// 接口日志服务
        /// </summary>
        private static APILogsMongRepository apiLogDal = new APILogsMongRepository();

        #endregion

        /// <summary>
        /// 进入处理
        /// </summary>
        /// <param name="actionContext"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public override async Task OnActionExecutingAsync(HttpActionContext actionContext, CancellationToken cancellationToken)
        {
            #region 获取传入的数据

            var jsonData = "";

            try
            {
                if (System.Web.HttpContext.Current.Request.ContentType != null && System.Web.HttpContext.Current.Request.ContentType != "")
                {
                    var contentType = System.Web.HttpContext.Current.Request.ContentType.ToLower();
                    if ((contentType.Contains("application/x-www-form-urlencoded")
                        || contentType.Contains("multipart/form-data"))
                        && System.Web.HttpContext.Current.Request.ContentLength > 0)
                    {
                        var bodyinfo = new Dictionary<string, string>();
                        foreach (string key in System.Web.HttpContext.Current.Request.Form)
                        {
                            bodyinfo.Add(key, System.Web.HttpContext.Current.Request.Form.GetValues(key)[0]);
                        }
                        jsonData = Newtonsoft.Json.JsonConvert.SerializeObject(bodyinfo);
                    }
                    else if (contentType.Contains("application/json"))
                    {
                        if (actionContext.ActionArguments.Count > 1)
                            jsonData = "[";
                        foreach (var item in actionContext.ActionArguments)
                        {
                            if (actionContext.ActionArguments.Count > 1)
                                jsonData += Newtonsoft.Json.JsonConvert.SerializeObject(item.Value) + ",";
                            else
                                jsonData = Newtonsoft.Json.JsonConvert.SerializeObject(item.Value);
                            continue;
                        }
                        if (actionContext.ActionArguments.Count > 1)
                            jsonData = jsonData.Substring(0, (jsonData.Length - 1)) + "]";

                    }
                    if (System.Web.HttpContext.Current.Request.Path.ToLower().IndexOf("ocr_result") > -1)
                    {
                        long big = Encoding.Default.GetByteCount(jsonData);
                        if (big > 40720)
                            jsonData = "";
                    }
                }

                #endregion

                #region 新增接口日志

                //获取全局日志追踪ID
                var globalLogTraceID = string.Empty;
                if (System.Web.HttpContext.Current.Request.Headers.GetValues(nameof(APILogConfig.PoJun_GlobalLogTraceID)) != null)
                    globalLogTraceID = System.Web.HttpContext.Current.Request.Headers.GetValues(nameof(APILogConfig.PoJun_GlobalLogTraceID)).FirstOrDefault();

                //获取全局日志级别
                int globalLogLevel = 0;
                if (System.Web.HttpContext.Current.Request.Headers.GetValues(nameof(APILogConfig.PoJun_GlobalLogLevel)) != null)
                    globalLogLevel = Convert.ToInt32(System.Web.HttpContext.Current.Request.Headers.GetValues(nameof(APILogConfig.PoJun_GlobalLogLevel)).FirstOrDefault());
                else
                    globalLogLevel++;

                //获取日志ID
                var logTraceID = IdHelp.GetGuidBy32();
                var requestTime = DateTime.Now;

                var logParam = new AddRequestLogParam();
                logParam.APIName = System.Web.HttpContext.Current.Request.Path;
                //这里存储的是客户端的IP+端口
                logParam.ClientHost = $"{System.Web.HttpContext.Current.Request.UserHostAddress}";
                var result = new Dictionary<string, string>();
                var headers = System.Web.HttpContext.Current.Request.Headers;
                foreach (string key in headers.Keys)
                {
                    result.Add(key, headers.GetValues(key)[0]);
                }
                logParam.Headers = Newtonsoft.Json.JsonConvert.SerializeObject(result);
                //logParam.Headers = Newtonsoft.Json.JsonConvert.SerializeObject(System.Web.HttpContext.Current.Request.Headers);
                logParam.Level = globalLogLevel;
                if (logParam.APIName.ToLower().IndexOf("uploadfile") > -1)  //如果上传文件流就清空
                    jsonData = "";
                logParam.RequestBody = jsonData;
                //这里存储的当前服务器的IP+端口+接口地址+url参数
                logParam.ServerHost = $"{System.Web.HttpContext.Current.Request.Url.AbsoluteUri}";
                logParam.SystemID = SysUtil.GetSystemId();
                logParam.TraceID = logTraceID;
                //如果全局日志追踪ID为空则使用本次日志追踪ID为全局日志追踪ID
                logParam.GlobalTraceID = string.IsNullOrEmpty(globalLogTraceID) ? logTraceID : globalLogTraceID;
                logParam.ParentTraceID = globalLogTraceID;
                logParam.RequestTime = requestTime;

                //新增日志
                await apiLogDal.AddRequestLogAsync(logParam);

                if (KafkaConfig.GetInstance().IsEnable)
                {
                    ////开启异步新线程把日志信息推送到kafka
                    //Task task = new Task(async () =>
                    //{
                    //    var kafkaLog = new KafkaLog();
                    //    kafkaLog.Body = Newtonsoft.Json.JsonConvert.SerializeObject(logParam);
                    //    kafkaLog.Type = 1;
                    //    await kafkaProducer.SendAsync(KafkaConfig.GetInstance().BrokerUrl, KafkaConfig.GetInstance().Topic, Newtonsoft.Json.JsonConvert.SerializeObject(kafkaLog));
                    //});
                    //task.Start();//执行
                }

                #endregion

                #region 向后传输数据

                //把日志ID写入
                System.Web.HttpContext.Current.Request.Headers.Add(nameof(APILogConfig.PoJun_LogTraceID), logTraceID);
                //把接口请求时间写入
                System.Web.HttpContext.Current.Request.Headers.Add(nameof(APILogConfig.RequestTime), requestTime.ToString("yyyy-MM-dd HH:mm:ss.fff"));
                //把全局日志级别写入
                System.Web.HttpContext.Current.Request.Headers.Remove(nameof(APILogConfig.PoJun_GlobalLogLevel));
                System.Web.HttpContext.Current.Request.Headers.Add(nameof(APILogConfig.PoJun_GlobalLogLevel), globalLogLevel.ToString());
                //如果全局日志追踪ID为空则使用本次日志追踪ID为全局日志追踪ID
                if (string.IsNullOrEmpty(globalLogTraceID))
                {
                    System.Web.HttpContext.Current.Request.Headers.Add(nameof(APILogConfig.PoJun_GlobalLogTraceID), logTraceID);
                }

                #endregion

                await base.OnActionExecutingAsync(actionContext, cancellationToken);
            }
            catch (Exception ex)
            {
                jsonData = $"参数获取失败报错！StackTrace:{ex.StackTrace} | Message:{ex.Message} | Source:{ex.Source}";
            }
        }
    }
}
