﻿using ApiLog.Act.DAL.Mongo;
using ApiLog.Act.Models.Log;
using ApiLog.Act.Tools;
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Web.Http.Filters;

namespace ApiLog.Act.Filters
{
    /// <summary>
    /// 返回数据日志处理
    /// </summary>
    public class ResponseLogAttribute : System.Web.Http.Filters.ActionFilterAttribute
    {
        #region 初始化

        /// <summary>
        /// 接口日志服务
        /// </summary>
        private static APILogsMongRepository apiLogDal = new APILogsMongRepository();



        #endregion

        /// <summary>
        /// 新增返回日志
        /// </summary>
        /// <param name="actionExecutedContext"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public override async Task OnActionExecutedAsync(HttpActionExecutedContext actionExecutedContext, CancellationToken cancellationToken)
        {
            #region 新增接口返回日志

            var logTraceID = System.Web.HttpContext.Current.Request.Headers.Get(nameof(APILogConfig.PoJun_LogTraceID));
            if (logTraceID != null)
            {
                var _requestTime = Convert.ToDateTime(System.Web.HttpContext.Current.Request.Headers.Get(nameof(APILogConfig.RequestTime)));
                var logParam = new AddResponseLogParam();
                logParam.IsError = false;

                if (actionExecutedContext.Response != null && System.Web.HttpContext.Current.Request.Path.ToLower().IndexOf("upload/showfile") <0)
                {
                    try
                    {
                        logParam.ResponseBody = await actionExecutedContext.Response.Content.ReadAsStringAsync();
                    }
                    catch (Exception ex)
                    {
                        logParam.ResponseBody = $"参数获取失败报错！StackTrace:{ex.StackTrace} | Message:{ex.Message} | Source:{ex.Source}";
                    }
                }
                else
                {
                    if (actionExecutedContext.Exception != null)
                    {
                        logParam.ErrorBody = $"StackTrace:{actionExecutedContext.Exception.StackTrace} | Message:{actionExecutedContext.Exception.Message} | Source:{actionExecutedContext.Exception.Source}";
                        logParam.IsError = true;
                    }
                }

                var _globalTraceID = System.Web.HttpContext.Current.Request.Headers.Get(nameof(APILogConfig.PoJun_GlobalLogTraceID));
                logParam.GlobalTraceID = _globalTraceID;
                logParam.ParentTraceID = logTraceID.ToString();
                logParam.ResponseTime = DateTime.Now;
                logParam.TimeCost = Convert.ToInt32((logParam.ResponseTime - Convert.ToDateTime(_requestTime)).TotalMilliseconds);
                await Task.Run(() =>
                {
                    apiLogDal.AddResponseLog(logParam);
                });
            }

            #endregion

            await base.OnActionExecutedAsync(actionExecutedContext, cancellationToken);
        }
    }
}
